function talMat = bredequery_lancaster_mni2tal(varargin)
%Converts coordinates from MNI brain to best guess for equivalent Talairach
%coordinates using the best-fit MTT transforms defined by Lancaster et al.
%in the article "Bias Metween MNI and Talairach Coordinates Analyzed Using
%the ICBM-152 Brain Template" from Human Brain Mapping 28:1194-1205
%(2007).
%
% FORMAT talMat = bredequery_lancaster_mni2tal(inpoints) 
% or
% talMat = bredequery_lancaster_mni2tal(inpoints, type) 
%
% Where inpoints is N by 3 or 3 by N matrix of coordinates
%  (N being the number of points)
% type is a type of MTT transform
% By default the MTTspm transform type is used when the second argument is not
% specified. Second argument can vary through:
%   'spm' - best-fit transformation for SPM2
%   'fsl' - best-fit transformation for FSL
%   'pooled' - best-fit transform after pooling SPM2 and FSL data.
%
% talMat is the coordinate matrix with Talairach points
%
% Bartlomiej Wilkowski
% Informatics and Mathematical Modelling
% Technical University of Denmark
% bw@imm.dtu.dk
% 2007/11/08
% Last modified: 2007/11/19 - bw


MTTspm = [0.9254 0.0024 -0.0118 -1.0207;-0.0048 0.9316 -0.0871 -1.7667;0.0152 0.0883 0.8924 4.0926;0 0 0 1];
MTTfsl = [0.9464 0.0034 -0.0026 -1.0680;-0.0083 0.9479 -0.0580 -1.0239;0.0053 0.0617 0.9010 3.1883;0 0 0 1];
MTTpooled = [0.9357 0.0029 -0.0072 -1.0423;-0.0065 0.9396 -0.0726 -1.3940;0.0103 0.0752 0.8967 3.6475;0 0 0 1];
if(nargin == 1)
    type = 'spm';
else if(nargin > 1)
        type = varargin{2};
    end 
end
n = size(varargin{1},1);
mniMat = [varargin{1} ones(n,1)];

if(strcmp(type,'spm'))
    val = MTTspm*mniMat';
else if(strcmp(type,'fsl'))
        val = MTTfsl*mniMat';
    else if(strcmp(type,'pooled'))
            val = MTTpooled*mniMat';
        else
            error('Incorrect MTT type! You can choose between [spm], [fsl] or [pooled]!');    
        end
    end
end

talMat = val(1:3,:)';