function bredequery_web(varargin)

%BREDEQUERY_WEB  Query the Brede Database from SPM.
%   This function makes a query to the Brede Database and displays the
%   response in a separate window. The results are displayed in the same
%   form as at the webpage:
%   http://hendrix.imm.dtu.dk/services/jerne/brede/ 
%
%   The input parameters are Talairach coordinates. It can be single
%   values or horizontal vectors of values. In case of vectors, all of
%   them must have the same number of elements. 
%
%   In case of vectors, an experimental seach is also queried for all
%   coordinates together.
%
%   The author is Bartlomiej Wilkowski, Department of Informatics and
%   Mathematical Modelling, Technical University of Denmark,
%   bw@imm.dtu.dk  
%
%   See also MNI2TAL.
%  
%   Copyright 2007-2008 Technical University of Denmark
%   $Revision: 1.1 $  $Date: 2008/01/25 11:56:12 $

import java.lang.String;
import java.util.Vector;

pointsNum = size(varargin{1}(:),1);
if(nargin == 3 || nargin == 4)
    x = varargin{1};
    y = varargin{2};
    z = varargin{3};
    if(pointsNum>1)
        %multiple points query (singular + experimental)
        expSearchQuery = String('');
        for i=1:pointsNum           
           bredequery_web(x(i), y(i), z(i));           
           expSearchQuery = expSearchQuery.concat('+').concat(num2str(x(i))).concat('+').concat(num2str(y(i))).concat('+').concat(num2str(z(i))).concat('%3B%0D%0A');
        end
        if ((nargin == 4) && varargin{4})
            bredequery_execute(expSearchQuery,1);
        end        
    else if(pointsNum == 1)
            %single point query
            bredeQuery = String([num2str(x) '+' num2str(y) '+' num2str(z)]);
            if ((nargin == 4) && varargin{4})
                bredequery_execute(bredeQuery,1);
            end
            bredequery_execute(bredeQuery,0);
        end        
    end
else if(nargin == 1 || nargin == 2)
        %query as a text string
        coords = Vector();
        coords = separate_points(String(varargin{1}),coords,';');
        vctSize = coords.size();
        x = zeros(1,vctSize,'double');
        y = zeros(1,vctSize,'double');
        z = zeros(1,vctSize,'double');
        for i=1:(vctSize)
            arr = extract_coords(coords.elementAt(i-1));
            arr_int = round(arr);
            x(i) = arr_int(1);
            y(i) = arr_int(2);
            z(i) = arr_int(3);
        end
        if(vctSize==1)
            if(nargin >1)
                bredequery_web(x(1),y(1),z(1),varargin{2});
            else
                bredequery_web(x(1),y(1),z(1),0);
            end
        else
            if(nargin >1)
                bredequery_web(x,y,z,varargin{2});
            else
                bredequery_web(x,y,z,0);
            end
        end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function bredequery_execute(query, exp)

import java.lang.String;

queryBaseAddress = String('http://hendrix.imm.dtu.dk/cgi-bin/brede_loc_query.pl?q=');
queryBaseAddressExperimental = String('http://hendrix.imm.dtu.dk/cgi-bin/brede_exp_query.pl?q=');

if(exp)
    queryURL = queryBaseAddressExperimental.concat(query);
else
    queryURL = queryBaseAddress.concat(query);
end

web(char(queryURL), '-new');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function arr = separate_points(query, coords, separator)

import java.lang.String;
import java.util.Vector;

query = String(query);
semicolon = query.indexOf(separator);

if (semicolon ~= -1) 
    coord = query.substring(0, semicolon);
    rest = query.substring(semicolon + 1);
    coords.add(coord);
    coords = separate_points(rest, coords, separator);
else
    if (query.length() > 0)
        coords.add(query);
    end
end

arr = coords;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function arr = extract_coords(point)

import java.lang.*;
point = String(point);
point = point.trim();

if (point.indexOf(',') ~= -1)
    separator = ',';
else
    separator = ' ';
end
coords = zeros(1,3,'double');
coords = coords-1;
for i=1:3
    try
        if (i < 3) 
            point = point.trim();
            coordinate = point.indexOf(separator);
            singleCoord = point.substring(0, coordinate);
            point = point.substring(coordinate + 1);
            
        else
            singleCoord = point;
        end
        val = Double(singleCoord);
        coords(1,i) = val;
   catch
       error('Incorrect coordinate data OR Incorrectly formatted query!');        
   end
end
arr = coords;



