package imm.brede.plugin;

import imm.brede.plugin.bibtex.BibTeXArticle;
import imm.brede.plugin.endnote.EndNoteArticle;
import imm.brede.plugin.refman.RefManJournal;
import imm.brede.plugin.refworks.RefWorksJournal;

import java.util.Vector;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * 
 * @author Bartlomiej Wilkowski Technical University of Denmark, Informatics and
 *         Mathematical Modelling, 2800 Kogens, Lyngby bw@imm.dtu.dk
 * 
 * Creation date: 2007-11-28
 * 
 */
public class BredeQueryXML {

	/**
	 * This method retrieves the main node (<Locs>) from the BredeXML. In
	 * BredeXML there is always one external <Locs> node which includes all the
	 * other elements/nodes of the document.
	 * 
	 * @param document -
	 *            parsed BredeXML document
	 * @return a reference to the main <Locs> node, null if <Locs> node does not
	 *         exist.
	 */
	public static Node getMainNode(Document document) {

		Node mainNode = document.getFirstChild();
		return mainNode;
	}

	/**
	 * This method retrieves all the <nodeName> nodes from the reference node.
	 * 
	 * @param refNode
	 *            reference node
	 * @param nodeName
	 *            name of the node to retrieve
	 * @return vector with the references to the <Loc> nodes
	 */
	public static Vector<Node> getAllNodes(Node refNode, String nodeName) {
		Vector<Node> list = new Vector<Node>();

		NodeList children = refNode.getChildNodes();
		Node temp;
		for (int i = 0; i < children.getLength(); i++) {
			temp = children.item(i);
			if (temp.getNodeName().equals(nodeName)) {
				list.add(temp);
			}
		}

		return list;
	}

	/**
	 * 
	 * @param parent
	 * @param nodeName
	 * @return
	 */
	public static Node getNode(Node parent, String nodeName) {

		NodeList children = parent.getChildNodes();
		Node temp;
		for (int i = 0; i < children.getLength(); i++) {
			temp = children.item(i);
			if (temp.getNodeName().equals(nodeName)) {
				return temp;
			}
		}

		return null;
	}

	/**
	 * 
	 * @param bib
	 * @param indexNum
	 * @param articleType -
	 *            defines the type of the article. 'b' - BibTeX ; 'r' -
	 *            Reference Manager (RIS); 'e' - EndNote; 
	 * @return
	 */
	public static Article getArticle(Node bib, int indexNum, char articleType) {

		Article bibArt;
		switch(articleType){
		case 'b': bibArt = new BibTeXArticle(); break;
		case 'r': bibArt = new RefManJournal(); break;
		case 'e': bibArt = new EndNoteArticle(); break;
		case 'w': bibArt = new RefWorksJournal(); break;
		default: bibArt = new BibTeXArticle();
		}
		

		NodeList bibList = bib.getChildNodes();
		String name;
		Node temp;
		for (int i = 0; i < bibList.getLength(); i++) {
			temp = bibList.item(i);
			name = temp.getNodeName();
			bibArt.setId(indexNum);
			if (name.equals(XMLArticleBibField.title)) {
				bibArt.setTitle(temp.getTextContent());
			} else if (name.equals(XMLArticleBibField.journal)) {
				bibArt.setJournal(temp.getTextContent());
			} else if (name.equals(XMLArticleBibField.authors)) {
				String authors = temp.getTextContent();
				Vector<String> authorVct = BredeUtils.extractAuthors(authors);
				bibArt.setAuthors(authorVct);
			} else if (name.equals(XMLArticleBibField.year)) {
				bibArt.setYear(temp.getTextContent());
			} else if (name.equals(XMLArticleBibField.volume)) {
				bibArt.setVolume(temp.getTextContent());
			} else if (name.equals(XMLArticleBibField.number)) {
				bibArt.setNumber(temp.getTextContent());
			} else if (name.equals(XMLArticleBibField.pages)) {
				bibArt.setPages(temp.getTextContent());
			} else if (name.equals(XMLArticleBibField.wobib)) {
				bibArt.setWobib(temp.getTextContent());
			}

		}
		return bibArt;
	}

}
