package imm.brede.plugin;

import imm.brede.plugin.bibtex.BibTeXArticle;
import imm.brede.plugin.bibtex.BibTeXWriter;
import imm.brede.plugin.endnote.EndNoteArticle;
import imm.brede.plugin.endnote.EndNoteWriter;
import imm.brede.plugin.refman.RefManJournal;
import imm.brede.plugin.refman.RefManWriter;
import imm.brede.plugin.refworks.RefWorksJournal;
import imm.brede.plugin.refworks.RefWorksWriter;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Date;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

/**
 * 
 * @author Bartlomiej Wilkowski Technical University of Denmark, Informatics and
 *         Mathematical Modelling, 2800 Kogens, Lyngby bw@imm.dtu.dk
 * 
 * Creation date: 2007-11-26
 * 
 */
public class BredeXMLConverter {

	static Document document;

	private static final String LOC = "Loc";
	private static final String BIB = "Bib";

	private String filesPath = "";
	private String fdate;
	private String query = "";
	private String algorithm = "";

	private File xmlFile = null;

	public BredeXMLConverter(String path, String quer, String alg) {

		this.query = quer;
		this.algorithm = alg;

		String date = new String(new Date(System.currentTimeMillis())
				.toString());

		this.fdate = date.substring(4, 7) + "_" + date.substring(8, 10) + "_"
				+ date.substring(date.length() - 4, date.length()) + "-"
				+ date.substring(11, 13) + "." + date.substring(14, 16);

		this.filesPath = path + "/BredeQuery-" + fdate;
		File tmp = new File(filesPath);
		if (!tmp.exists()) {
			tmp.mkdirs();
		}

	}

	/**
	 * 
	 */
	public void createInfoFile() {
		File infoFile = new File(filesPath + "/BredeQueryInfo.txt");
		try {
			FileOutputStream fos = new FileOutputStream(infoFile);
			PrintWriter pw = new PrintWriter(fos);
			pw
					.println("This is an information file about the query made from SPM to Brede database using BredeQuery plugin.");
			pw.println("\n\nCreation date:" + fdate);
			String space = "";
			if (algorithm.equals("none"))
				space = "MNI";
			else
				space = "Talairach";
			pw.println("\n\nQueried coordinates in " + space + " space: " + query);
			if (space.equals("Talairach"))
				pw
						.println("Coordinates were converted from MNI space to Talairach using the algorithm: "
								+ algorithm);
			
			pw.flush();
			pw.close();
			fos.close();
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * 
	 * @param query
	 * @return
	 */
	public int getBredeXML() {

		int[] coords = BredeUtils.extract3DCoordinates(query);
		if (coords == null)
			return 0;

		String bredeQuery = coords[0] + "+" + coords[1] + "+" + coords[2];
		String finalQuery = Brede.queryBaseAddress + bredeQuery
				+ Brede.resultToXML;
		System.out.println(finalQuery);

		URL xmlResults;
		xmlFile = new File(filesPath + "/BredeQuery.xml");

		try {
			xmlResults = new URL(finalQuery);
			URLConnection httpConn = xmlResults.openConnection();
			BufferedReader dis = new BufferedReader(new InputStreamReader(
					httpConn.getInputStream()));

			String inputLine;
			FileOutputStream fos = new FileOutputStream(xmlFile);
			PrintWriter pw = new PrintWriter(fos);
			pw
					.println("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>");
			while ((inputLine = dis.readLine()) != null) {
				pw.println(inputLine);
			}
			pw.flush();
			pw.close();
			fos.close();
			dis.close();
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return 1;
	}

	/**
	 * 
	 * @param xmlFile
	 */
	public void convertToRefWorks(File xmlFile) {

		if (parseXML(xmlFile)) {
			Node locsNode = BredeQueryXML.getMainNode(document);

			Vector<Node> locNodes = BredeQueryXML.getAllNodes(locsNode, LOC);

			Vector<RefWorksJournal> refworks = retrieveAllRefWorksArticles(locNodes);
			RefWorksWriter.createRefWorksFile(refworks, new File(filesPath
					+ "/BredeQuery.rw"));
		}
	}

	/**
	 * 
	 * @param nodes
	 * @return
	 */
	private Vector<RefWorksJournal> retrieveAllRefWorksArticles(
			Vector<Node> nodes) {

		Vector<RefWorksJournal> articles = new Vector<RefWorksJournal>();

		Node tempBib;
		RefWorksJournal bibArt;
		for (int i = 0; i < nodes.size(); i++) {
			tempBib = BredeQueryXML.getNode(nodes.elementAt(i), BIB);
			bibArt = (RefWorksJournal) BredeQueryXML.getArticle(tempBib, i + 1,
					'w');
			articles.add(bibArt);
		}

		return articles;
	}

	/**
	 * 
	 * @param xmlFile
	 */
	public void convertToEndNote(File xmlFile) {

		if (parseXML(xmlFile)) {
			Node locsNode = BredeQueryXML.getMainNode(document);

			Vector<Node> locNodes = BredeQueryXML.getAllNodes(locsNode, LOC);

			Vector<EndNoteArticle> refmans = retrieveAllEndNoteArticles(locNodes);
			EndNoteWriter.createEndNoteFile(refmans, new File(filesPath
					+ "/BredeQuery.enw"));
		}
	}

	/**
	 * 
	 * @param nodes
	 * @return
	 */
	private Vector<EndNoteArticle> retrieveAllEndNoteArticles(Vector<Node> nodes) {

		Vector<EndNoteArticle> articles = new Vector<EndNoteArticle>();

		Node tempBib;
		EndNoteArticle bibArt;
		for (int i = 0; i < nodes.size(); i++) {
			tempBib = BredeQueryXML.getNode(nodes.elementAt(i), BIB);
			bibArt = (EndNoteArticle) BredeQueryXML.getArticle(tempBib, i + 1,
					'e');
			articles.add(bibArt);
		}

		return articles;
	}

	/**
	 * 
	 * @param xmlFile
	 */
	public void convertToRefMan(File xmlFile) {

		if (parseXML(xmlFile)) {
			Node locsNode = BredeQueryXML.getMainNode(document);

			Vector<Node> locNodes = BredeQueryXML.getAllNodes(locsNode, LOC);

			Vector<RefManJournal> endnotes = retrieveAllRefManArticles(locNodes);
			RefManWriter.createRefManFile(endnotes, new File(filesPath
					+ "/BredeQuery.ris"));
		}
	}

	/**
	 * 
	 * @param nodes
	 * @return
	 */
	private Vector<RefManJournal> retrieveAllRefManArticles(Vector<Node> nodes) {

		Vector<RefManJournal> articles = new Vector<RefManJournal>();

		Node tempBib;
		RefManJournal bibArt;
		for (int i = 0; i < nodes.size(); i++) {
			tempBib = BredeQueryXML.getNode(nodes.elementAt(i), BIB);
			bibArt = (RefManJournal) BredeQueryXML.getArticle(tempBib, i + 1,
					'r');
			articles.add(bibArt);
		}

		return articles;
	}

	/**
	 * 
	 * @param xmlFile
	 */
	public void convertToBibTeX(File xmlFile) {

		if (parseXML(xmlFile)) {
			Node locsNode = BredeQueryXML.getMainNode(document);

			Vector<Node> locNodes = BredeQueryXML.getAllNodes(locsNode, LOC);

			Vector<BibTeXArticle> bibs = retrieveAllBibTeXArticles(locNodes);
			BibTeXWriter.createBibTeXFile(bibs, new File(filesPath
					+ "/BredeQuery.bib"));
		}
	}

	/**
	 * 
	 * @param nodes
	 * @return
	 */
	private Vector<BibTeXArticle> retrieveAllBibTeXArticles(Vector<Node> nodes) {

		Vector<BibTeXArticle> articles = new Vector<BibTeXArticle>();

		Node tempBib;
		BibTeXArticle bibArt;
		for (int i = 0; i < nodes.size(); i++) {
			tempBib = BredeQueryXML.getNode(nodes.elementAt(i), BIB);
			bibArt = (BibTeXArticle) BredeQueryXML.getArticle(tempBib, i + 1,
					'b');
			articles.add(bibArt);
		}

		return articles;
	}

	/**
	 * 
	 * @param xmlFile
	 * @return
	 */
	private boolean parseXML(File xmlFile) {
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		// factory.setValidating(true);
		// factory.setNamespaceAware(true);
		try {
			DocumentBuilder builder = factory.newDocumentBuilder();
			document = builder.parse(xmlFile);

		} catch (SAXException sxe) {
			// Error generated during parsing)
			Exception x = sxe;
			if (sxe.getException() != null)
				x = sxe.getException();
			x.printStackTrace();
			return false;

		} catch (ParserConfigurationException pce) {
			// Parser with specified options can't be built
			pce.printStackTrace();
			return false;

		} catch (IOException ioe) {
			// I/O error
			ioe.printStackTrace();
			return false;
		}
		return true;
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		BredeXMLConverter bb = new BredeXMLConverter("C:/Bartek", "48 -40 9",
				"none");
		if (bb.getBredeXML() != 0) {
			bb.createInfoFile();
			bb.convertToBibTeX(bb.getXmlFile());
			bb.convertToRefMan(bb.getXmlFile());
			bb.convertToEndNote(bb.getXmlFile());
			bb.convertToRefWorks(bb.getXmlFile());
		}
	}

	public File getXmlFile() {
		return xmlFile;
	}

	public void setXmlFile(File xmlFile) {
		this.xmlFile = xmlFile;
	}

}
