package imm.brede.plugin.old;
import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;

/**
 * 
 * @author Bartlomiej Wilkowski
 * Technical University of Denmark,
 * Informatics and Mathematical Modelling,
 * 2800 Kogens, Lyngby
 * bw@imm.dtu.dk
 * 
 * Creation date: 2007-10-15
 *
 */
public class BredeQueryResult extends JFrame {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private String title;
	private URL queryUrl;
	private int disp = 0;

	private JPanel jContentPane = null;
	private JScrollPane jScrollPane = null;
	private JEditorPane jEditorPane = null;

	public BredeQueryResult(String title, String url) {
		this(title, url, 0);
	}
		
		
	
	public BredeQueryResult(String title, String url, int displacement) {
		this.title = title;
		this.disp = displacement;
		try {
			this.queryUrl = new URL(url);
		} catch (MalformedURLException e) {
			JOptionPane.showMessageDialog(null, "Argh! Bad url: " + queryUrl);
			e.printStackTrace();
			return;
		}
		initialize();
	}

	private void initialize() {
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		setBounds(new Rectangle(80+disp, 15+disp, 400, 400));
		setPreferredSize(new Dimension(400, 400));
		setResizable(true);
		setContentPane(getJContentPane());
		setTitle(title);
		
	}

	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.setPreferredSize(new Dimension(400, 400));
			jContentPane.add(getJScrollPane(), BorderLayout.CENTER);
		}
		return jContentPane;
	}

	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane(getEditorPane());
			jScrollPane.setPreferredSize(new Dimension(400, 400));
			jScrollPane
					.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
			jScrollPane
					.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		}
		return jScrollPane;
	}

	private JEditorPane getEditorPane() {
		if (jEditorPane == null) {
			try {
				jEditorPane = new JEditorPane(queryUrl);
				jEditorPane.setEditable(false);
				jEditorPane.addHyperlinkListener(new HyperlinkListener() {

					public void hyperlinkUpdate(HyperlinkEvent e) {
						if (e.getEventType().equals(
								HyperlinkEvent.EventType.ACTIVATED)) {
							URL url = e.getURL();
							try {
								jEditorPane.setCursor(new Cursor(Cursor.WAIT_CURSOR));
								jEditorPane.setPage(url);
								jEditorPane.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
							} catch (IOException e1) {
								JOptionPane.showMessageDialog(null,
										"Argh! Bad url: " + url);
								e1.printStackTrace();
							}
						}

					}

				});
			} catch (IOException e) {
				JOptionPane.showMessageDialog(null, "Argh! Bad url: "
						+ queryUrl);
				e.printStackTrace();
			}
		}
		return jEditorPane;
	}

	public static void main(String[] args) {
		BredeQueryResult bqr = new BredeQueryResult("Brede",
				"http://hendrix.imm.dtu.dk/cgi-bin/brede_loc_query.pl?q=-12+450+-21");
		bqr.setVisible(true);

	}

}
