package imm.brede.plugin.old;

import imm.brede.plugin.Brede;

import java.util.Vector;

import javax.swing.JOptionPane;

/**
 * 
 * @author Bartlomiej Wilkowski
 * Technical University of Denmark,
 * Informatics and Mathematical Modelling,
 * 2800 Kogens, Lyngby
 * bw@imm.dtu.dk
 * 
 * Creation date: 2007-10-15
 * 
 */
public class SPMBredePlugin {

	private int disp = 0;
	private int counter = 0;

	/**
	 * This method shows the Brede database retrieved data for points defined
	 * through Talairach coordinates given in the arrays.
	 * 
	 * @param x -
	 *            array of Talairach x coordinates
	 * @param y -
	 *            array of Talairach y coordinates
	 * @param z -
	 *            array of Talairach z coordinates
	 */
	public void getBredeInfo(int[] x, int[] y, int[] z) {
		int length = x.length;
		String expSeachQuery = "";
		String expSearchString = "Experimental search";
		if (y.length != length || z.length != length) {
			JOptionPane.showMessageDialog(null,
					"The arrays of coordinates must have the same size!");
			return;
		}
		for (int i = 0; i < length; i++) {
			disp = counter * 25;
			getBredeInfo(x[i], y[i], z[i]);
			counter++;
			expSeachQuery = expSeachQuery + "+" + x[i] + "+" + y[i] + "+"
					+ z[i] + "%3B%0D%0A";
		}

		/* experimental search */
		disp = counter * 25;
		executeQuery(expSeachQuery, expSearchString, true);

	}

	/**
	 * This method show the Brede database retrieved data for single point
	 * defined through Talairach coordinates.
	 * 
	 * @param x -
	 *            single x Talairach coordinate
	 * @param y -
	 *            single y Talairach coordinate
	 * @param z -
	 *            single z Talairach coordinate
	 */
	public void getBredeInfo(int x, int y, int z) {

		String bredeQuery = x + "+" + y + "+" + z;
		String bredeQueryCoordinates = x + "," + y + "," + z;
		executeQuery(bredeQuery, bredeQueryCoordinates, false);
	}

	public void getBredeInfo(String query) {
		Vector<String> coords = new Vector<String>();
		coords = separatePoints(query, coords, ';');
		int size = coords.size();
		/*for (int i = 0; i < size; i++) {
			System.out.println(i + ": " + coords.elementAt(i));
		}*/
		int[] x = new int[size];
		int[] y = new int[size];
		int[] z = new int[size];
		double[] point = new double[3];
		for (int i = 0; i < size; i++) {
			point = extractCoordinates(coords.elementAt(i));
			if (point == null)
				return;
			x[i] = (int) point[0];
			y[i] = (int) point[1];
			z[i] = (int) point[2];
		}
		/*for (int i = 0; i < x.length; i++) {
			System.out.println(x[i] + " " + y[i] + " " + z[i]);
		}*/
		if (size == 1)
			getBredeInfo(x[0], y[0], z[0]);
		else
			getBredeInfo(x, y, z);

	}

	private double[] extractCoordinates(String point) {
		point = point.trim();
		char separator;
		if (point.indexOf(',') != -1)
			separator = ',';
		else
			separator = ' ';

		double[] coords = new double[3];
		int coordinate;
		String singleCoord = "";
		for (int i = 0; i < 3; i++) {
			try {
				if (i < 2) {
					coordinate = point.indexOf(separator);
					singleCoord = point.substring(0, coordinate);
					point = point.substring(coordinate + 1);
				} else {
					singleCoord = point;
				}
				coords[i] = new Double(singleCoord).doubleValue();
			} catch (NumberFormatException e) {
				JOptionPane.showMessageDialog(null,
						"Incorrect coordinate data. Entered value is not a number. Check it: \n\n\t"
								+ singleCoord);
				return null;
			} catch (Exception e) {
				JOptionPane.showMessageDialog(null,
						"Incorrectly formatted query.");
				return null;
			}
		}
		return coords;
	}

	private Vector<String> separatePoints(String query, Vector<String> coords,
			char separator) {

		int semicolon = query.indexOf(separator);
		if (semicolon != -1) {
			String coord = query.substring(0, semicolon);
			String rest = query.substring(semicolon + 1);
			coords.add(coord);
			coords = separatePoints(rest, coords, separator);
		} else {
			if (query.length() > 0)
				coords.add(query);
		}

		return coords;
	}

	private void executeQuery(String bredeQuery, String bredeQueryCoordinates,
			boolean experimental) {
		String bredeUrl;
		if (experimental)
			bredeUrl = Brede.queryBaseAddressExperimental + bredeQuery;
		else
			bredeUrl = Brede.queryBaseAddress + bredeQuery;
		System.out.println(bredeUrl);
		String queryTitle = Brede.queryWindowTitle + bredeQueryCoordinates;
		BredeQueryResult bqr = new BredeQueryResult(queryTitle, bredeUrl, disp);
		bqr.setVisible(true);
	}

	public static void main(String[] args) {
		SPMBredePlugin spm = new SPMBredePlugin();
		/*
		 * spm.getBredeInfo(new int[] { 1, 10, -15 }, new int[] { -20, 35, 45 },
		 * new int[] { 14, -78, 56 });
		 */
		spm.getBredeInfo("23 45 12; 22 33 1");
	}

}
